@@ -1,6 +1,8 @@
 package Mojo::Redis::PubSub;
 use Mojo::Base 'Mojo::EventEmitter';
 
+use Mojo::JSON qw(from_json to_json);
+
 use constant DEBUG => $ENV{MOJO_REDIS_DEBUG};
 
 has connection => sub { shift->redis->_connection };
@@ -12,6 +14,8 @@ sub channels_p {
   shift->db->call_p(qw(PUBSUB CHANNELS), @_);
 }
 
+sub json { ++$_[0]{json}{$_[1]} and return $_[0] }
+
 sub keyspace_listen {
   my ($self, $cb) = (shift, pop);
   return $self->listen($self->_keyspace_key(@_), $cb);
@@ -32,7 +36,9 @@ sub listen {
 }
 
 sub notify {
-  shift->db->call_p(PUBLISH => @_);
+  my ($self, $name, $payload) = @_;
+  $payload = to_json $payload if $self->{json}{$name};
+  shift->db->call_p(PUBLISH => $name, $payload);
 }
 
 sub numsub_p {
@@ -81,11 +87,12 @@ sub _listen {
   my $reconnecting = delete $self->{reconnecting};
 
   $self->{setup} = 1;
-  $conn->once(close => sub { $self->_reconnect->emit(disconnect => shift) });
+  $conn->once(close => sub { $self and $self->_reconnect->emit(disconnect => shift) });
   $conn->on(
     response => sub {
       my ($conn, $res) = @_;    # $res = [$type, $name, $payload]
       return unless ref $res eq 'ARRAY' and @$res >= 3;
+      $res->[2] = eval { from_json $res->[2] } if $self->{json}{$res->[1]};
       for my $cb (@{$self->{chans}{$res->[1]}}) { $self->$cb($res->[2]) }
     }
   );
@@ -214,6 +221,20 @@ Holds a L<Mojo::Redis> object used to create the connections to talk with Redis.
 Lists the currently active channels. An active channel is a Pub/Sub channel
 with one or more subscribers (not including clients subscribed to patterns).
 
+=head2 json
+
+  $pubsub = $pubsub->json("foo");
+
+Activate automatic JSON encoding and decoding with L<Mojo::JSON/"to_json"> and
+L<Mojo::JSON/"from_json"> for a channel.
+
+  # Send and receive data structures
+  $pubsub->json("foo")->listen(foo => sub {
+    my ($pubsub, $payload) = @_;
+    say $payload->{bar};
+  });
+  $pubsub->notify(foo => {bar => 'I ♥ Mojolicious!'});
+
 =head2 keyspace_listen
 
   $cb = $pubsub->keyspace_listen($key, $op, sub { my ($pubsub, $message) = @_ }) });
